import { Subject } from 'rxjs';
export const NOTIFICATION_SUBJECT = {
    open$: new Subject(),
    close$: new Subject(),
    rerender$: new Subject(),
    closeAll$: new Subject(),
};
class Notification {
    constructor(_uniqueId) {
        Object.defineProperty(this, "_uniqueId", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: _uniqueId
        });
    }
    get uniqueId() {
        return this._uniqueId;
    }
    close() {
        NotificationService.close(this._uniqueId);
    }
    rerender(props) {
        NotificationService.rerender(this._uniqueId, props);
    }
}
let NOTIFICATIONS = [];
let UNIQUEID = 1;
export class NotificationService {
    static get notifications() {
        return NOTIFICATIONS;
    }
    static open(props) {
        UNIQUEID += 1;
        NOTIFICATION_SUBJECT.open$.next({ uniqueId: UNIQUEID, props });
        const notification = new Notification(UNIQUEID);
        NOTIFICATIONS.push(notification);
        return notification;
    }
    static close(uniqueId) {
        NOTIFICATIONS = NOTIFICATIONS.filter((item) => item.uniqueId !== uniqueId);
        NOTIFICATION_SUBJECT.close$.next(uniqueId);
    }
    static rerender(uniqueId, props) {
        NOTIFICATION_SUBJECT.rerender$.next({ uniqueId, props });
    }
    static closeAll(animation = true) {
        NOTIFICATIONS = [];
        NOTIFICATION_SUBJECT.closeAll$.next(animation);
    }
}
