import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { useEffect } from 'react';
import ReactDOM from 'react-dom';
import { useImmer, useRefExtra } from '@react-devui/hooks';
import { NotificationService, NOTIFICATION_SUBJECT } from '../notification';
import { DNotification } from '../notification/Notification';
import { usePrefixConfig } from './hooks';
export function Notification() {
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    const [notifications, setNotifications] = useImmer(new Map());
    const getRoot = (id) => {
        let root = document.getElementById(`${dPrefix}notification-root`);
        if (!root) {
            root = document.createElement('div');
            root.id = `${dPrefix}notification-root`;
            document.body.appendChild(root);
        }
        let el = document.getElementById(id);
        if (!el) {
            el = document.createElement('div');
            el.id = id;
            root.appendChild(el);
        }
        return el;
    };
    const notificationLTRootRef = useRefExtra(() => getRoot(`${dPrefix}notification-lt-root`), true);
    const notificationRTRootRef = useRefExtra(() => getRoot(`${dPrefix}notification-rt-root`), true);
    const notificationLBRootRef = useRefExtra(() => getRoot(`${dPrefix}notification-lb-root`), true);
    const notificationRBRootRef = useRefExtra(() => getRoot(`${dPrefix}notification-rb-root`), true);
    useEffect(() => {
        const obs = [];
        const mergeProps = (uniqueId, props) => {
            return {
                onClose: () => {
                    var _a;
                    (_a = props.onClose) === null || _a === void 0 ? void 0 : _a.call(props);
                    NotificationService.close(uniqueId);
                },
                afterVisibleChange: (visible) => {
                    var _a;
                    (_a = props.afterVisibleChange) === null || _a === void 0 ? void 0 : _a.call(props, visible);
                    if (!visible) {
                        setNotifications((draft) => {
                            draft.delete(uniqueId);
                        });
                    }
                },
            };
        };
        obs.push(NOTIFICATION_SUBJECT.open$.subscribe({
            next: ({ uniqueId, props }) => {
                setNotifications((draft) => {
                    draft.set(uniqueId, Object.assign(Object.assign(Object.assign({}, props), { dVisible: true }), mergeProps(uniqueId, props)));
                });
            },
        }), NOTIFICATION_SUBJECT.close$.subscribe({
            next: (uniqueId) => {
                setNotifications((draft) => {
                    const props = draft.get(uniqueId);
                    if (props) {
                        draft.set(uniqueId, Object.assign(Object.assign({}, props), { dVisible: false }));
                    }
                });
            },
        }), NOTIFICATION_SUBJECT.rerender$.subscribe({
            next: ({ uniqueId, props: newProps }) => {
                setNotifications((draft) => {
                    const props = draft.get(uniqueId);
                    if (props) {
                        draft.set(uniqueId, Object.assign(Object.assign(Object.assign({}, newProps), { dVisible: props.dVisible }), mergeProps(uniqueId, newProps)));
                    }
                });
            },
        }), NOTIFICATION_SUBJECT.closeAll$.subscribe({
            next: (animation) => {
                setNotifications((draft) => {
                    if (animation) {
                        for (const props of draft.values()) {
                            props.dVisible = false;
                        }
                    }
                    else {
                        draft.clear();
                    }
                });
            },
        }));
    }, [setNotifications]);
    return (_jsxs(_Fragment, { children: [notificationLTRootRef.current &&
                ReactDOM.createPortal(Array.from(notifications.entries())
                    .filter(([, notificationProps]) => notificationProps.dPlacement === 'left-top')
                    .map(([uniqueId, notificationProps]) => _jsx(DNotification, Object.assign({}, notificationProps), uniqueId)), notificationLTRootRef.current), notificationRTRootRef.current &&
                ReactDOM.createPortal(Array.from(notifications.entries())
                    .filter(([, notificationProps]) => { var _a; return ((_a = notificationProps.dPlacement) !== null && _a !== void 0 ? _a : 'right-top') === 'right-top'; })
                    .map(([uniqueId, notificationProps]) => _jsx(DNotification, Object.assign({}, notificationProps), uniqueId)), notificationRTRootRef.current), notificationLBRootRef.current &&
                ReactDOM.createPortal(Array.from(notifications.entries())
                    .filter(([, notificationProps]) => notificationProps.dPlacement === 'left-bottom')
                    .map(([uniqueId, notificationProps]) => _jsx(DNotification, Object.assign({}, notificationProps), uniqueId)), notificationLBRootRef.current), notificationRBRootRef.current &&
                ReactDOM.createPortal(Array.from(notifications.entries())
                    .filter(([, notificationProps]) => notificationProps.dPlacement === 'right-bottom')
                    .map(([uniqueId, notificationProps]) => _jsx(DNotification, Object.assign({}, notificationProps), uniqueId)), notificationRBRootRef.current)] }));
}
